
#include "client_activity.h"
#include <string>
#include <thread>

namespace sample
{
    uint8_t const app_index = ClientActivity::CAMERA_A;
    void ClientActivity::CameraA_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
                                                     ara::com::FindServiceHandle handler)
    {
        log_.LogInfo() << "SOC_A cam serviceAvailabilityCallback.";
        for (auto it : handles)
        {
            std::lock_guard<std::mutex> lock(proxySocAMutex_[CAMERA_A]);
            if (it.GetInstanceId().ToString() == "1" && proxySocA_[CAMERA_A] == nullptr)
            {
                log_.LogInfo() << "Created camProxySocA_ instance ";
                proxySocA_[CAMERA_A] = std::make_shared<ProxySocA>(it);
                proxySocA_[CAMERA_A]->CameraH265ImagePDataViz.SetReceiveHandler([this]() { ClientActivity::CameraA_ImageCallback(); });
                const size_t maxSampleCount = 10U;
                proxySocA_[CAMERA_A]->CameraH265ImagePDataViz.Subscribe(maxSampleCount);
            }
        }
    }

    void ClientActivity::CameraA_ImageCallback()
    {
        log_.LogInfo() << "CameraImage event on soc_a received.";

        proxySocA_[CAMERA_A]->CameraH265ImagePDataViz.GetNewSamples([this](ara::com::SamplePtr<CameraH265Image const> ptr)
                                                    {
        auto sample = ptr.Get();
        mdc::visual::Image image;
        image.frameData = sample->rawData;
        image.header.frameId = sample->header.frameId;
        image.header.stamp.sec = sample->header.stampLow;
        image.header.stamp.nsec = sample->header.stampHigh;

		CameraA_imagePub[sample->cameraId].Publish(image);

        log_.LogInfo() << "camera id: " << sample->cameraId; });
    }


    void ClientActivity::CameraA_setupPublishers()
    {
        for (int i = 0; i < 4; i++)
        {
            std::string topic = topic_prefix + std::to_string(i + soc_a_cam_idx_div_factor);
            CameraA_imagePub[i] = mdc::visual::Publisher::Advertise<mdc::visual::Image>(topic);
            // imagePub  = mdc::visual::Publisher::Advertise<mdc::visual::Image>("test");
        }
    }

    void ClientActivity::CameraA_setupFindService()
    {
        serviceHandleA[app_index] = ProxySocA::StartFindService(
        [this](ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles, ara::com::FindServiceHandle handler)
        {
            ClientActivity::CameraA_ServiceAvailabilityCallback(std::move(handles), handler);
        },
        ara::core::InstanceSpecifier(
            "visualization/visualization/CameraH265ImageRPort"));
    }

    void ClientActivity::CameraA_stop()
    {
        if (proxySocA_[CAMERA_A] != nullptr && proxySocA_[CAMERA_A]->CameraH265ImagePDataViz.IsSubscribed())
        {
        	proxySocA_[CAMERA_A]->CameraH265ImagePDataViz.UnsetReceiveHandler();
        	proxySocA_[CAMERA_A]->CameraH265ImagePDataViz.Unsubscribe();
        }
        ProxySocA::StopFindService(serviceHandleA[app_index]);
    }

}
